function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? Object(arguments[i]) : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

/**
 * original html structure (note that extra divs have been inserted by using fela)
 *
 * <div class="cf-select">
 *   <label class="cf-select__label">Label</label>
 *
 *   <div class="Select is-searchable has-value">
 *     <input type="hidden"/>
 *
 *     <div class="Select-control">
 *       <div class="Select-placeholder">Placeholder</div>
 *
 *       <!-- When not searchable: -->
 *       <div class="Select-input" tabindex="0">&nbsp;</div>
 *
 *       <!-- When searchable: -->
 *       <div class="Select-input" style="...">
 *         <input value="" tabindex="0" style="...">
 *         <div style="..."></div>
 *       </div>
 *
 *       <!-- When searchable and has option -->
 *       <span class="Select-clear-zone" title="Clear value" aria-label="Clear value">
 *         <span class="Select-clear">×</span>
 *       </span>
 *
 *       <div class="Select-arrow-zone">
 *         <span class="Select-arrow"></span>
 *       </div>
 *     </div>
 *
 *     <!-- only when open -->
 *     <div class="Select-menu-outer">
 *       <div class="Select-menu">
 *         <div class="Select-option is-selected">One</div>
 *         <div class="Select-option">Two</div>
 *         <div class="Select-option is-focused">Three</div>
 *       </div>
 *     </div>
 *
 *   </div>
 * </div>
 */
import React from 'react';
import PropTypes from 'prop-types';
import ReactSelect, { AsyncCreatable, Async, Creatable } from 'react-select';
import { createComponent, createStyledComponent } from '@cloudflare/style-container';
import { isDarkMode } from '@cloudflare/style-const';
import { Trans } from '@cloudflare/intl-react';
import Value from './Value';
import Option from './Option';
import MenuHeader from './MenuHeader';
import CloseMenu from './CloseMenu';
import menuRenderer from './menuRenderer';
import defaultArrowRenderer from './arrowRenderer';
import actionArrowRenderer, { actionArrowRendererInvert } from './actionArrowRenderer';
import clearRenderer from './clearRenderer';
const Label = createComponent(({
  theme
}) => ({
  display: 'block',
  fontSize: theme.fontSizes[2],
  color: theme.colors.gray[1],
  marginBottom: theme.space[1]
}), 'label'); // bindRight is used to ensure that menuRenderer, called from within react-select,
// is always passed a second parameter which is used for further configuration.

const bindRight = (fn, ...rightArgs) => (...args) => fn(...args, ...rightArgs);

class Select extends React.Component {
  constructor(props) {
    super(props);
    this.menuRenderer = bindRight(props.menuRenderer, {
      menuTitle: props.menuTitle,
      menuHeaderComponent: props.menuHeaderComponent,
      closeMenuComponent: props.closeMenuComponent,
      closeMenu: () => this.select.closeMenu()
    });
  }

  render() {
    const _this$props = this.props,
          async = _this$props.async,
          creatable = _this$props.creatable,
          searchable = _this$props.searchable,
          label = _this$props.label,
          className = _this$props.className,
          multi = _this$props.multi,
          clearable = _this$props.clearable,
          notClearable = _this$props.notClearable,
          onChange = _this$props.onChange,
          testId = _this$props.testId,
          action = _this$props.action,
          invert = _this$props.invert,
          arrowRenderer = _this$props.arrowRenderer,
          props = _objectWithoutProperties(_this$props, ["async", "creatable", "searchable", "label", "className", "multi", "clearable", "notClearable", "onChange", "testId", "action", "invert", "arrowRenderer"]);

    let SelectClass;

    if (async) {
      SelectClass = creatable ? AsyncCreatable : Async;
    } else {
      SelectClass = creatable ? Creatable : ReactSelect;
    } // this logic sucks, but it's the only way to be backwards compatible
    // and work for this use case -- https://jira.cfops.it/browse/CUSTESC-2094


    const isClearable = !notClearable && (searchable || creatable || multi || clearable); // Ref access differs depending on whether creatable is used

    props[creatable ? 'innerRef' : 'ref'] = ref => this.select = ref;

    const handleChange = e => {
      if (onChange) {
        onChange(e);
      }
    };

    return /*#__PURE__*/React.createElement("div", {
      className: `${className}`,
      "data-testid": testId
    }, label && /*#__PURE__*/React.createElement(Label, null, label), /*#__PURE__*/React.createElement(SelectClass, _extends({}, props, {
      arrowRenderer: arrowRenderer === undefined ? action ? invert ? actionArrowRendererInvert : actionArrowRenderer : defaultArrowRenderer : arrowRenderer,
      onChange: handleChange,
      searchable: searchable,
      clearable: isClearable,
      backspaceRemoves: isClearable,
      menuRenderer: this.menuRenderer,
      multi: multi
    })));
  }

}

Select.defaultProps = {
  multi: false,
  searchable: false,
  async: false,
  creatable: false,
  clearable: false,
  notClearable: false,
  disabled: false,
  menuRenderer,
  clearRenderer,
  optionComponent: Option,
  valueComponent: Value,
  menuHeaderComponent: MenuHeader,
  closeMenuComponent: CloseMenu,
  // Using the `_` prop allows us to default to text when used outside of dash.
  placeholder: /*#__PURE__*/React.createElement(Trans, {
    id: "common.select",
    _: "Select..."
  })
};
Select.propTypes = {
  className: PropTypes.string,
  disabled: PropTypes.bool,
  label: PropTypes.node,
  value: PropTypes.any,
  options: PropTypes.array,
  searchable: PropTypes.bool,
  multi: PropTypes.bool,
  onBlur: PropTypes.func,
  onChange: PropTypes.func,
  onOpen: PropTypes.func,
  async: PropTypes.bool,
  creatable: PropTypes.bool,
  menuRenderer: PropTypes.func,
  arrowRenderer: PropTypes.oneOfType([PropTypes.func, PropTypes.bool]),
  clearRenderer: PropTypes.func,
  isValidNewOption: PropTypes.func,
  noResultsText: PropTypes.string,
  promptTextCreator: PropTypes.func,
  newOptionCreator: PropTypes.func,
  menuTitle: PropTypes.string,
  menuHeaderComponent: PropTypes.func,
  closeMenuComponent: PropTypes.func,
  loadOptions: PropTypes.func,
  autoload: PropTypes.bool,
  searchPromptText: PropTypes.string,
  cache: PropTypes.bool,
  clearable: PropTypes.bool,
  notClearable: PropTypes.bool,
  placeholder: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  testId: PropTypes.string,
  action: PropTypes.bool,
  invert: PropTypes.bool,
  onBlurResetsInput: PropTypes.bool,
  onCloseResetsInput: PropTypes.bool,
  invalid: PropTypes.bool
};

const singleOrPlaceholder = (theme, isPlaceholder, action, invert) => ({
  bottom: 0,
  color: action ? invert ? theme.colors.white : theme.colors.blue[isDarkMode() ? 3 : 4] : theme.colors.gray[isPlaceholder ? 4 : 1],
  left: 0,
  lineHeight: '32px',
  paddingLeft: '10px',
  paddingRight: '10px',
  position: 'absolute',
  right: 0,
  top: 0,
  maxWidth: 'calc(100% - 29px)',
  overflow: 'hidden',
  textOverflow: 'ellipsis',
  whiteSpace: 'nowrap'
});

const styles = ({
  theme,
  disabled,
  action,
  invert,
  invalid
}) => {
  const values = {
    boxSizing: 'border-box',
    fontSize: theme.fontSizes[2]
  };
  return _objectSpread({}, values, {
    '& div': values,
    '& input': values,
    '& span': values,
    flex: 'initial',
    margin: 'initial',
    width: '100%',
    display: 'initial',
    verticalAlign: 'initial',
    position: 'relative',
    textAlign: 'left',
    '& .Select-control': {
      backgroundColor: disabled ? theme.colors.gray[8] : action && invert ? theme.colors.blue[isDarkMode() ? 3 : 4] : theme.colors.white,
      border: '1px solid',
      borderColor: invalid ? theme.colors.red[3] : action ? theme.colors.blue[isDarkMode() ? 3 : 4] : theme.colors.gray[isDarkMode() ? 4 : 5],
      borderRadius: theme.radii[2],
      color: action && invert ? theme.colors.white : theme.colors.gray[1],
      cursor: disabled ? 'default' : 'pointer',
      display: 'table',
      borderSpacing: 0,
      borderCollapse: 'separate',
      height: '35px',
      outline: 'none',
      overflow: 'hidden',
      position: 'relative',
      tableLayout: 'fixed',
      width: '100%',
      transition: 'border-color 0.2s ease',
      opacity: disabled ? 0.5 : 1,
      paddingBottom: theme.space[1]
    },
    '& .Select-control .Select-input:focus': {
      outline: 'none'
    },
    '& .Select-menu-outer': {
      position: 'relative'
    },
    '&:hover .Select-control': {
      borderColor: invalid ? theme.colors.red[3] : disabled ? theme.colors.gray[isDarkMode() ? 4 : 5] : theme.colors.blue[isDarkMode() ? 3 : 4]
    },
    '& .is-open .Select-control': {
      borderColor: action ? theme.colors.blue[isDarkMode() ? 3 : 4] : theme.colors.gray[isDarkMode() ? 4 : 5]
    },
    '& .is-focused .Select-control': _objectSpread({
      borderColor: invalid ? theme.colors.red[3] : theme.colors.blue[isDarkMode() ? 3 : 4]
    }, action && !invert ? {
      border: `2px solid ${theme.colors.blue[isDarkMode() ? 3 : 4]}`
    } : {}, {
      boxShadow: `${theme.colors.background} 0px 0px 0px 3px, ${invalid ? theme.colors.red[3] : theme.colors.blue[isDarkMode() ? 3 : 4]} 0px 0px 0px 5px`
    }),
    '& .Select-placeholder': singleOrPlaceholder(theme, true, action, invert),
    '& .is-focused.is-searchable .Select-placeholder': {
      display: 'none'
    },
    '& .Select--single > .Select-control .Select-value': singleOrPlaceholder(theme, false),
    '& .Select--multi > .Select-control .Select-value': {
      padding: '0 5px 1px 5px',
      minHeight: '23px',
      display: 'flex',
      flexDirection: 'row-reverse',
      alignItems: 'center'
    },
    '& .Select--multi > .Select-control .Select-value-label': {
      padding: '0px',
      top: '0px',
      overflowWrap: 'break-word',
      wordBreak: 'break-all'
    },
    '& .Select--multi > .Select-control .Select-value-icon': {
      padding: '0px 5px 1px 5px'
    },
    '& .Select-input': {
      verticalAlign: 'middle',
      overflow: 'hidden',
      maxWidth: 'calc(100%)',
      marginTop: '4px'
    },
    '& .Select-input > input': {
      color: theme.colors.gray[2],
      width: '100%',
      background: 'none transparent',
      border: ' 0 none',
      cursor: 'default',
      display: 'inline-block',
      fontFamily: 'inherit',
      fontSize: 'inherit',
      height: '100%',
      marginLeft: theme.space[2],
      marginRight: theme.space[2],
      marginTop: 0,
      marginBottom: 0,
      outline: 'none',
      padding: 0,
      transition: 'none'
    },
    '& .Select-noresults': {
      boxVizing: 'border-box',
      color: theme.colors.gray[4],
      cursor: 'default',
      display: 'block',
      padding: theme.space[2]
    },
    '& .Select-aria-only': {
      border: 0,
      clip: 'rect(0, 0, 0, 0)',
      clipPath: 'inset(50%)',
      height: 'auto',
      width: '1px',
      margin: 0,
      padding: 0,
      overflow: 'hidden',
      position: 'absolute',
      whiteSpace: 'nowrap'
    },
    '& .Select-arrow-zone': {
      cursor: disabled ? 'default' : 'pointer',
      display: 'table-cell',
      position: 'relative',
      textAlign: 'center',
      verticalAlign: 'middle',
      width: '25px',
      paddingRight: theme.space[1],
      top: '1px',
      pointerEvents: 'none'
    },
    '& .Select-clear-zone': {
      color: theme.colors.gray[4],
      cursor: ' pointer',
      display: 'table-cell',
      position: 'relative',
      textAlign: 'center',
      verticalAlign: 'middle',
      width: '17px'
    }
  });
};

Select.displayName = 'Select';
export default createStyledComponent(styles, Select);