import React from 'react';
import PropTypes from 'prop-types';
import { createComponent } from '@cloudflare/style-container';
import ScrollContainer from './ScrollContainer';
const MenuWrapper = createComponent(({
  theme
}) => {
  return {
    border: `1px solid ${theme.colors.gray[5]}`,
    borderRadius: theme.borderRadius,
    backgroundColor: theme.colors.white,
    marginTop: '5px',
    boxSizing: 'border-box',
    position: 'absolute',
    top: '100%',
    minWidth: '100%',
    maxWidth: '350px',
    zIndex: '1100',
    webkitOverflowScrolling: 'touch'
  };
});
export default function menuRenderer({
  focusedOption,
  instancePrefix,
  onFocus,
  onSelect,
  optionComponent,
  optionRenderer,
  options,
  valueArray,
  valueKey,
  onOptionRef
}, {
  menuTitle,
  closeMenu,
  closeMenuComponent,
  menuHeaderComponent
}) {
  const MenuHeader = menuHeaderComponent;
  const CloseMenu = closeMenuComponent;
  const Option = optionComponent;
  const opts = options.map((option, i) => {
    let isSelected = valueArray && valueArray.indexOf(option) > -1;
    let isFocused = option === focusedOption;
    return /*#__PURE__*/React.createElement(Option, {
      instancePrefix: instancePrefix,
      isDisabled: option.disabled,
      isFocused: isFocused,
      isSelected: isSelected,
      onFocus: onFocus,
      onSelect: onSelect,
      option: option,
      optionIndex: i,
      ref: _ref => {
        onOptionRef(_ref, isFocused);
      },
      key: `option-${i}-${option[valueKey]}`
    }, optionRenderer(option, i));
  });
  return /*#__PURE__*/React.createElement(MenuWrapper, null, menuTitle && /*#__PURE__*/React.createElement(MenuHeader, null, /*#__PURE__*/React.createElement("span", null, menuTitle), /*#__PURE__*/React.createElement(CloseMenu, {
    closeMenu: closeMenu
  })), /*#__PURE__*/React.createElement(ScrollContainer, null, opts));
}
menuRenderer.propTypes = {
  focusedOption: PropTypes.object,
  instancePrefix: PropTypes.string,
  onFocus: PropTypes.func,
  onSelect: PropTypes.func,
  optionComponent: PropTypes.element,
  optionRenderer: PropTypes.func,
  options: PropTypes.array,
  valueArray: PropTypes.array,
  valueKey: PropTypes.string,
  onOptionRef: PropTypes.func
};